# copyright (C) 1997-98 Jean-Luc Fontaine (mailto:jfontain@mygale.org)
# this program is free software: please refer to the BSD type license enclosed in this package

set rcsId {$Id: datapie.tcl,v 1.4 1998/03/10 22:35:01 jfontain Exp $}

class dataPieChart {

    proc dataPieChart {this parentPath width height thickness args} composite {[new canvas $parentPath] $args} viewer {} {
        set path $widget::($this,path)

        set dataPieChart::($this,pie) [new pie $path 0 0 $width $height\
            -title {} -thickness $thickness -background gray\
            -labeller [new piePeripheralLabeller $path -font $font::(mediumNormal) -smallfont $font::(smallNormal)]\
        ]
        set dataPieChart::($this,cells) {}
        set dataPieChart::($this,slices) {}

        # allow dropping of data cells
        set dataPieChart::($this,drop) [new dropSite\
            -path $path -formats DATACELL -command "viewer::view $this \$dragSite::data(DATACELL)"\
        ]
        set dataPieChart::($this,drag) [new dragSite -path $path]
        dragSite::provide $dataPieChart::($this,drag) OBJECTS "dataPieChart::dragData $this"            ;# provide object itself

        bind $path <Configure> "dataPieChart::resize $this"

        composite::complete $this
    }

    proc ~dataPieChart {this} {
        catch {trace vdelete $dataPieChart::($this,array)(updates) w "dataPieChart::update $this"}        ;# eventually remove trace
        delete $dataPieChart::($this,pie) $dataPieChart::($this,drag) $dataPieChart::($this,drop)
    }

    proc options {this} {
        # force size values
        return [list\
            [list -height height Height 200]\
            [list -width width Width 300]\
        ]
    }

    foreach option {-height -width} {
        proc set$option {this value} "\$widget::(\$this,path) configure $option \$value"
    }

    proc dragData {this format} {                                                   ;# only OBJECTS format is supported at this time
        return $this
    }

    proc supportedTypes {this} {
        return {integer real}
    }

    proc monitorCell {this array row column} {
        if {![info exists dataPieChart::($this,array)]} {                                   ;# the module array should stay the same
            trace variable ${array}(updates) w "dataPieChart::update $this"                   ;# refresh as module data gets updated
            set dataPieChart::($this,array) $array
        }
        set newCell ${array}($row,$column)
        foreach cell $dataPieChart::($this,cells) {
            if {[string compare $cell $newCell]==0} return                                                 ;# already charted, abort
        }
        set label "[set ${array}($row,0)] [set ${array}($column,label)]"
        lappend dataPieChart::($this,slices) [pie::newSlice $dataPieChart::($this,pie) $label]
        lappend dataPieChart::($this,cells) $newCell
    }

    proc update {this args} {                                    ;# update display using cells data. ignore eventual trace arguments
        set pie $dataPieChart::($this,pie)
        set sum 0.0                                                                             ;# force floating point calculations
        foreach cell $dataPieChart::($this,cells) {
            catch {set sum [expr {$sum+[set $cell]}]}                              ;# ignore errors as data cell may no longer exist
        }
        foreach slice $dataPieChart::($this,slices) cell $dataPieChart::($this,cells) {
            if {[catch {set $cell} value]||($sum==0)} {                            ;# handle invalid cells and divide by zero errors
                pie::sizeSlice $pie $slice 0 ?
            } else {
                set value [expr {[set $cell]/$sum}]
                pie::sizeSlice $pie $slice $value "[format %.1f [expr {$value*100}]] %"
            }
        }
        resize $this
    }

    proc resize {this} {                                                                   ;# make sure all canvas items are visible
        $widget::($this,path) configure -scrollregion [$widget::($this,path) bbox all]
    }
}

class data2DPieChart {

    proc data2DPieChart {this parentPath args} dataPieChart {$parentPath 160 160 0 -width 200 -height 220 $args} {}

    proc ~data2DPieChart {this} {}

    proc iconData {} {
        return {
            R0lGODdhKAAoAPYAAHt5e87Pzv///wAA/wAAANbX1hAoCAAIADFhITFhGHNJAAAQAK33e6X3e5zn
            c6Xvc/ffSpzna+/XQpTXa4zPY+fHOZTfazk4OUJJQnvHWta2MYTHWs6mKYTPY3u+Wnu+UlI4ABAI
            ABgQADEgACkgACEYABBhe63X55TH3oy+1jF5lO/PQmumvVKWree+OSEQAN6+Md62MVIIY2sYe2sQ
            e4wQpb0o1jkoAHMoe6UYvbUYzq0Yzr0Y3t44/7Ug1kIoAM6eIWsge95B/wAYIQAQEBBphHMwe+dJ
            /7Xf76XP5wAQGOdR/4S+znOuxudZ/9YY984Y7wAgKedh/wAoOQAwQkKOpd4o/+9x/3M4ewAAAAAA
            AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
            AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACwA
            AAAAKAAoAAAH/oAAgoOEhYaHiIMBAASNjo+QkZKTjYsEAZiZAQWYnJuanp6ao6OMAQKoqagAqq2s
            ramvq6iWp7ACsrC5rqqvlgW3uMHCwbm+pomECMsICs7JhrXFqMsN1ggQ2RDOvLG0jMC3ANXW19ra
            3LPeAtK3COUNDg4N2OcS9wrq6rWH7/DyFOrdG1ihgoJEv2D5swaQQsCB9wpKPNjr26VWC+M5cOgg
            A4KIEjWIFJlvXUJV/uRtpLDyowSJIUlWZIds0EKVHR16hBlTAwcOJXEJakctnsYMGXTuLDiy6c+g
            xwiEE5Cy4UqkCJg29fnz6T5Tqd6pZJmUAlKPTruqBfr1ks14/jrNKkW7ti5QDISIvuM492wzZ4AD
            B2YVdSoCvmbPnjVxorFjxyhSpFBhgrBFW1QbJFWsmPFjyJJZtKBsmaZUm0g7ckbq+XGKE6FHU8ar
            CGxY1atbw24cWvToyqWJUl29mDdvySlaKFdBOrjtsMR1I0/Bwjdz4G0jIYjueHry5dclWZIhY4Z5
            89s5e57uWzbl8/BnWKpBvz799KxPRKZeXbl7E/DZIKCAluSgg4EI6rADfiZU159/zF133g489NBD
            hT1YsoOCHG64w4IINPige9cBaN4OQqSoohAafujiix+aYEKENFJm4okpHqHjjhrasIOPQApBIQ88
            zCDjkUfCXPfhikxaQqQQPEC5hBBLVGnlDvFN+KGVXHLp5BI8TClml13CSOaZSzjpxJpstunmm3DG
            ackTUtRp55145qnnnnPu6eefflpihRVXFGrooYgmqqiii5Di6KOQQhoIADs=
        }
    }

}

class data3DPieChart {

    proc data3DPieChart {this parentPath args} dataPieChart {$parentPath 200 100 20 -width 250 -height 180 $args} {}

    proc ~data3DPieChart {this} {}

    proc iconData {} {
        return {
            R0lGODdhKAAoAPUAAHt5e87Pzv///wAA/wAAANbX1jFhIXNJAKX3e/ffSu/XQufHOZznc9a2MYzP
            Y86mKXvHWhBhe63X55TH3lKWrYy+1jF5lEJJQmumvVIIY2sYe4wQpb0o1qUYvbUYzr0Y3t44/60Y
            zt5B/+dJ/+dR/+dZ/9YY9+dh/94o/+9x/wAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
            AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACwAAAAAKAAoAAAG/kCAcEgsGo/IYQBA
            aDqf0Kh02lwSAthsoIDlbrVer3Y8ZgYE6DQaoG6z2+n3Gm09wwVyeN6tflsLd3iBgoF5fmZJRAaL
            BgeOB4lFdYMCjAiXBgmaCY99ngJ/gYuXpAiZmwkKC45zcXRmdwalpAynmqoLCw0HhK11RrKzDAwO
            trm6DcmQR6FqwbTDDsWpx8jJu5+Tac8IDtHS07nX4w0PvL5MgNvQxN7SBtbkD/PnhL9Cz9Ht4Aby
            8//0JMFCE0zaN2kQIPQDyPDfuUME1FW6dBBhwkaPMmp8+OrKtm/7EiaMIKGkyZITJkigQCECuitD
            DLhrJ3LkSZMpJ1SoQMFC/oQLRLRVIlazKMmbOXey9MkGosRFRY2arJBUac8IEZp2tFPJQFSRRyVU
            ncCyp8+s9tLFVPgVAsmxZJf69AlUycCubKOS3Mm3rIW/EQBrBXXXq2G9ff3+nSs4rcdtbKGOtFDW
            7GLGTAdbgQLVMKPAlwFjBQ04ipUMGTSo1tA58ujXpC9HWE3byobbuDe09ho7tOzVHIIHt9LBQ/Hj
            Hnb39u1z9QcQIJ6DsBLCQ/XrHpIfXi57tuoPIsKLpx6ivHnzHz4zH01b/Ij376lzCDG//of79zXA
            Zk8bvPj/IlhxnwjgfUCCCCQkqCBtq4XgnIIQQiggCQYiiGCEEIpwXggXOGIYoYAlhCjiiCSWaOKJ
            VphwwoostujiizDGmGKMNNZIoxUooJDCjjz26OOPQAK5BBlEFmmkkUEAADs=
        }
    }

}
